Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Prj_syntax.AxisSourceDefines axes for correct overlay with different Coordinate Systems. If not provided, the default values are the following:
"Lon", EAST, AXIS"Lat", NORTH"X", EAST, AXIS"Y", NORTH"X", OTHER, AXIS"Y", EAST, AXIS"Z", NORTHDefines axes for correct overlay with different Coordinate Systems. If not provided, the default values are the following:
"Lon", EAST, AXIS"Lat", NORTH"X", EAST, AXIS"Y", NORTH"X", OTHER, AXIS"Y", EAST, AXIS"Z", NORTH